import { Module, Provider } from '@nestjs/common';
import { CqrsModule } from '@nestjs/cqrs';
import {
  AD_MESSAGE_PUBLISHER,
  AD_REPOSITORY,
  AD_DIRECTION_ENCODER,
  AD_ROUTE_PROVIDER,
  AD_GET_BASIC_ROUTE_CONTROLLER,
} from './ad.di-tokens';
import { MessageBrokerPublisher } from '@mobicoop/message-broker-module';
import { AdRepository } from './infrastructure/ad.repository';
import { PrismaService } from './infrastructure/prisma.service';
import { AdMapper } from './ad.mapper';
import { AdCreatedMessageHandler } from './interface/message-handlers/ad-created.message-handler';
import { PostgresDirectionEncoder } from '@modules/geography/infrastructure/postgres-direction-encoder';
import { GetBasicRouteController } from '@modules/geography/interface/controllers/get-basic-route.controller';
import { RouteProvider } from './infrastructure/route-provider';
import { GeographyModule } from '@modules/geography/geography.module';

const messageHandlers = [AdCreatedMessageHandler];

const mappers: Provider[] = [AdMapper];

const repositories: Provider[] = [
  {
    provide: AD_REPOSITORY,
    useClass: AdRepository,
  },
];

const messagePublishers: Provider[] = [
  {
    provide: AD_MESSAGE_PUBLISHER,
    useExisting: MessageBrokerPublisher,
  },
];

const orms: Provider[] = [PrismaService];

const adapters: Provider[] = [
  {
    provide: AD_DIRECTION_ENCODER,
    useClass: PostgresDirectionEncoder,
  },
  {
    provide: AD_ROUTE_PROVIDER,
    useClass: RouteProvider,
  },
  {
    provide: AD_GET_BASIC_ROUTE_CONTROLLER,
    useClass: GetBasicRouteController,
  },
];

@Module({
  imports: [CqrsModule, GeographyModule],
  providers: [
    ...messageHandlers,
    ...mappers,
    ...repositories,
    ...messagePublishers,
    ...orms,
    ...adapters,
  ],
  exports: [PrismaService, AdMapper, AD_REPOSITORY, AD_DIRECTION_ENCODER],
})
export class AdModule {}
