import { ArgumentOutOfRangeException } from '@mobicoop/ddd-library';
import { Point } from '@modules/ad/core/domain/value-objects/point.value-object';

describe('Point value object', () => {
  it('should create a point value object', () => {
    const pointVO = new Point({
      lon: 48.689445,
      lat: 6.17651,
    });
    expect(pointVO.lon).toBe(48.689445);
    expect(pointVO.lat).toBe(6.17651);
  });
  it('should throw an exception if longitude is invalid', () => {
    try {
      new Point({
        lon: 348.689445,
        lat: 6.17651,
      });
    } catch (e: any) {
      expect(e).toBeInstanceOf(ArgumentOutOfRangeException);
    }
    try {
      new Point({
        lon: -348.689445,
        lat: 6.17651,
      });
    } catch (e: any) {
      expect(e).toBeInstanceOf(ArgumentOutOfRangeException);
    }
  });
  it('should throw an exception if latitude is invalid', () => {
    try {
      new Point({
        lon: 48.689445,
        lat: 96.17651,
      });
    } catch (e: any) {
      expect(e).toBeInstanceOf(ArgumentOutOfRangeException);
    }
    try {
      new Point({
        lon: 48.689445,
        lat: -96.17651,
      });
    } catch (e: any) {
      expect(e).toBeInstanceOf(ArgumentOutOfRangeException);
    }
  });
});
